use axum::{
    extract::Json,
    response,
    routing::{get, post},
    Router,
};
use battlesnake::types::{
    simulation::Board,
    wire::{Request, Response},
    Direction,
};
use log::{debug, info, warn};
use rand::prelude::*;
use serde::Serialize;
use tokio::{net::TcpListener, time::Instant};

#[tokio::main]
async fn main() {
    env_logger::init();

    debug!("Creating routes");
    let app = Router::new()
        .route("/", get(info))
        .route("/start", post(start))
        .route("/move", post(get_move))
        .route("/end", post(end));

    debug!("Creating listener");
    let listener = TcpListener::bind("0.0.0.0:8000").await.unwrap();
    debug!("Starting server");
    axum::serve(listener, app).await.unwrap();
}

async fn info() -> response::Json<Info> {
    info!("got info request");
    response::Json(Info {
        apiversion: "1",
        author: "der-informatiker",
        color: "#00FFEE",
        head: "smart-caterpillar",
        tail: "mouse",
        version: env!("CARGO_PKG_VERSION"),
    })
}

#[derive(Debug, Clone, Serialize)]
struct Info {
    apiversion: &'static str,
    author: &'static str,
    color: &'static str,
    head: &'static str,
    tail: &'static str,
    version: &'static str,
}

async fn start(request: Json<Request>) {
    let board = Board::from(&*request);
    info!("got start request: {board}");
}

async fn get_move(request: Json<Request>) -> response::Json<Response> {
    let board = Board::from(&*request);
    info!("got move request: {board}");
    let actions = board.valid_actions(0).collect::<Vec<_>>();
    info!("valid actions: {:?}", actions);
    let start = Instant::now();
    for _ in 0..100 {
        let mut board = board.clone();
        let score = board.simulate_random(|board| (board.num_snakes() <= 1).then_some(1));
        std::hint::black_box(score);
    }
    let elapsed = start.elapsed();
    debug!("simulated 100 random games in {elapsed:?}");
    let action = actions.choose(&mut thread_rng()).copied();
    if action.is_none() {
        warn!("unable to find a valid action");
    }
    info!("chose {action:?}");
    response::Json(Response {
        direction: action.unwrap_or(Direction::Up),
        shout: None,
    })
}

async fn end(request: Json<Request>) {
    let board = Board::from(&*request);
    info!("got end request: {board}");
}
